class BaseCodeBlock():
    """
    This class includes base methods and attributes for all other CodeBlocks
    """
    def __init__(self):
        """
        Initializes the base code block.
        """
        pass
